#!/usr/bin/python
#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.

import sys
#reload(sys) 
#sys.setdefaultencoding("utf-8") 
#sys.path.append("..")

import getopt
import readline
import shlex

from os.path import expanduser

from pydnsshim.LoginRequest import LoginRequest
from pydnsshim.LogoutRequest import LogoutRequest
from pydnsshim.SetExpirationPeriodResponse import SetExpirationPeriodResponse
from pydnsshim.ZoneVersionResponse import ZoneVersionResponse
from pydnsshim.ListKeysResponse import ListKeysResponse
from pydnsshim.ListZonesResponse import ListZonesResponse
from pydnsshim.ListZonesBySlaveGroupResponse import ListZonesBySlaveGroupResponse
from pydnsshim.NewKeyResponse import NewKeyResponse
from pydnsshim.ImportKeyResponse import ImportKeyResponse
from pydnsshim.RemoveKeyResponse import RemoveKeyResponse
from pydnsshim.ChangeKeyStatusResponse import ChangeKeyStatusResponse
from pydnsshim.NewZoneResponse import NewZoneResponse
from pydnsshim.ImportZoneResponse import ImportZoneResponse
from pydnsshim.PrintZoneResponse import PrintZoneResponse
from pydnsshim.PubZoneResponse import PubZoneResponse
from pydnsshim.RemoveZoneResponse import RemoveZoneResponse
from pydnsshim.ListSlaveGroupResponse import ListSlaveGroupResponse
from pydnsshim.ListSlavesResponse import ListSlavesResponse
from pydnsshim.PrintSlaveGroupResponse import PrintSlaveGroupResponse
from pydnsshim.SlaveOperation import SlaveOperation
from pydnsshim.NewTsigKeyResponse import NewTsigKeyResponse
from pydnsshim.ListTsigKeysResponse import ListTsigKeysResponse
from pydnsshim.LoginResponse import LoginResponse
from pydnsshim.Response import Response
from pydnsshim.RrOperation import RrOperation
from pydnsshim.Transport import Transport, TransportException
from pydnsshim.ResponseStatus import ResponseStatus

from Dnssh import CommandParser
from Dnssh import CommandUsage
from Dnssh.CommandParserException import CommandParserException


SERVER = 'localhost'
PORT = 9999
HISTORY_FILE = expanduser("~")+ '/.dnssh_history'

class Dnssh:
	def __init__(self, server = SERVER, port = PORT, username = '', password = ''):
		self.server = server
		self.port = port
		self.sessionId = 0
		self.transport = Transport(self.server, self.port)
		self.username = ''
		self.password = ''
		if (username != '' and password != ''):
			self.username = username
			self.password = password
		
	def parseCommand(self, command, args):
		transport = self.transport
		sessionId = self.sessionId
		if (command == 'server'):
			if (len(args) < 1 or args[0] == ''):
				CommandUsage.usage(command)
				return ''
			
			self.server = args[0]
			self.transport = Transport(self.server, self.port)
			return 'server: ' + self.server
		
		elif (command == 'port'):
			if (len(args) < 1 or args[0] == ''):
				CommandUsage.usage(command)
				return ''
			
			self.port = int(args[0])
			self.transport = Transport(self.server, self.port)
			return 'port: ' + str(self.port)
			
		elif (command == 'login'):
			xml = CommandParser.parseLogin(args)
			responseXml = transport.sendRequest(xml)
			loginResponse = LoginResponse()
			loginResponse.parseXml(responseXml)
			if (loginResponse.status == ResponseStatus.OK):
				self.sessionId = loginResponse.sessionId
			return str(loginResponse)
		
		elif (command == 'logout'):
			xml = CommandParser.parseLogout(sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)			
		
		elif (command == 'hello'):
			xml = CommandParser.parseHello(sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
	
		elif (command == 'add-user'):
			xml = CommandParser.parseAddUser(args)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
		
		elif (command == 'change-password'):
			xml = CommandParser.parseChangePassword(args, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
		
		elif (command == 'new-zone'):
			xml = CommandParser.parseNewZone(args, sessionId)
			responseXml = transport.sendRequest(xml)
			newZoneResponse = NewZoneResponse()
			newZoneResponse.parseXml(responseXml)
			return str(newZoneResponse)
		
		elif (command == 'import-zone'):
			xml = CommandParser.parseImportZone(args, sessionId)
			responseXml = transport.sendRequest(xml)
			importZoneResponse = ImportZoneResponse()
			importZoneResponse.parseXml(responseXml)
			return str(importZoneResponse)
		
		elif (command == 'list-zones'):
			xml = CommandParser.parseListZones(sessionId)
			responseXml = transport.sendRequest(xml)
			listZonesResponse = ListZonesResponse()
			listZonesResponse.parseXml(responseXml)
			return str(listZonesResponse)
		
		elif (command == 'print-zone'):
			xml = CommandParser.parsePrintZone(args, sessionId)
			responseXml = transport.sendRequest(xml)
			printZoneResponse = PrintZoneResponse()
			printZoneResponse.parseXml(responseXml)
			return str(printZoneResponse)

		elif (command == 'zone-exists'):
			xml = CommandParser.parseZoneExists(args, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
		
		elif (command == 'add-rr' or command == 'remove-rr'):
			operation = RrOperation.ADD
			if (command == 'remove-rr'):
				operation = RrOperation.REMOVE
				
			xml = CommandParser.parseRrOperation(args, operation, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
		
		elif (command == 'pub-zone'):
			xml = CommandParser.parsePubZone(args, sessionId)
			responseXml = transport.sendRequest(xml)
			pubZoneResponse = PubZoneResponse()
			pubZoneResponse.parseXml(responseXml)
			return str(pubZoneResponse)
		
		elif (command == 'remove-zone'):
			xml = CommandParser.parseRemoveZone(args, sessionId)
			responseXml = transport.sendRequest(xml)
			removeZoneResponse = RemoveZoneResponse()
			removeZoneResponse.parseXml(responseXml)
			return str(removeZoneResponse)
		
		elif (command == 'zone-version'):
			xml = CommandParser.parseZoneVersion(args, sessionId)
			responseXml = transport.sendRequest(xml)
			zoneVersionResponse = ZoneVersionResponse()
			zoneVersionResponse.parseXml(responseXml)
			return str(zoneVersionResponse)
		
		elif (command == 'set-expiration-period'):
			xml = CommandParser.parseSetExpirationPeriod(args, sessionId)
			responseXml = transport.sendRequest(xml)
			setExpirationPeriodResponse = SetExpirationPeriodResponse()
			setExpirationPeriodResponse.parseXml(responseXml)
			return str(setExpirationPeriodResponse)
		
		elif (command == 'new-key'):
			xml = CommandParser.parseNewKey(args, sessionId)
			responseXml = transport.sendRequest(xml)
			response = NewKeyResponse()
			response.parseXml(responseXml)
			return str(response)
		
		elif (command == 'import-key'):
			xml = CommandParser.parseImportKey(args, sessionId)
			responseXml = transport.sendRequest(xml)
			importKeyResponse = ImportKeyResponse()
			importKeyResponse.parseXml(responseXml)
			return str(importKeyResponse)
		
		elif (command == 'list-keys'):
			xml = CommandParser.parseListKeys(args, sessionId)
			responseXml = transport.sendRequest(xml)
			listKeysResponse = ListKeysResponse()
			listKeysResponse.parseXml(responseXml)
			return str(listKeysResponse)
		
		elif (command == 'change-key-status'):
			xml = CommandParser.parseChangeKeyStatus(args, sessionId)
			responseXml = transport.sendRequest(xml)
			changeKeyResponse = ChangeKeyStatusResponse()
			changeKeyResponse.parseXml(responseXml)
			return str(changeKeyResponse)
		
		elif (command == 'remove-key'):
			xml = CommandParser.parseRemoveKey(args, sessionId)
			responseXml = transport.sendRequest(xml)
			removeKeyResponse = RemoveKeyResponse()
			removeKeyResponse.parseXml(responseXml)
			return str(removeKeyResponse)
				
		elif (command in ['new-slavegroup', 'remove-slavegroup',
										 'assign-slavegroup', 'unassign-slavegroup']):
			operation = SlaveOperation.ADD
			if (command == 'remove-slavegroup'):
				operation = SlaveOperation.REMOVE
			elif (command == 'assign-slavegroup'):
				operation = SlaveOperation.ASSIGN
			elif (command == 'unassign-slavegroup'):
				operation = SlaveOperation.UNASSIGN
				
			xml = CommandParser.parseSlaveGroupOperation(args, operation, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
	
		elif (command == 'list-slavegroups'):
			xml = CommandParser.parseListSlaveGroups(sessionId)
			responseXml = transport.sendRequest(xml)
			listSlaveGroupResponse = ListSlaveGroupResponse()
			listSlaveGroupResponse.parseXml(responseXml)
			return str(listSlaveGroupResponse)
		
		elif (command == 'list-zones-by-slavegroup'):
			xml = CommandParser.parseListZonesBySlavegroup(args, sessionId)
			responseXml = transport.sendRequest(xml)
			listZonesResponse = ListZonesBySlaveGroupResponse()
			listZonesResponse.parseXml(responseXml)
			return str(listZonesResponse)
		
		elif (command == 'print-slavegroup'):
			xml = CommandParser.parsePrintSlaveGroup(args, sessionId)
			responseXml = transport.sendRequest(xml)
			printSlaveGroupResponse = PrintSlaveGroupResponse()
			printSlaveGroupResponse.parseXml(responseXml)
			return str(printSlaveGroupResponse)

		elif (command in ['add-slave', 'remove-slave']):
			operation = SlaveOperation.ADD
			if (command == 'remove-slave'):
				operation = SlaveOperation.REMOVE
				
			xml = CommandParser.parseSlaveOperation(args, operation, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
	
		elif (command == 'list-slaves'):
			xml = CommandParser.parseListSlaves(args, sessionId)
			responseXml = transport.sendRequest(xml)
			listSlavesResponse = ListSlavesResponse()
			listSlavesResponse.parseXml(responseXml)
			return str(listSlavesResponse)

		elif (command == 'new-tsig-key'):
			xml = CommandParser.parseNewTsigKey(args, sessionId)
			responseXml = transport.sendRequest(xml)
			tsigKeyResponse = NewTsigKeyResponse()
			tsigKeyResponse.parseXml(responseXml)
			return str(tsigKeyResponse)
		
		elif (command == 'remove-tsig-key'):
			xml = CommandParser.parseRemoveTsigKey(args, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)
		
		elif (command == 'list-tsig-keys'):
			xml = CommandParser.parseListTsigKeys(args, sessionId)
			responseXml = transport.sendRequest(xml)
			listTsigKeysResponse = ListTsigKeysResponse()
			listTsigKeysResponse.parseXml(responseXml)
			return str(listTsigKeysResponse)
				
		elif (command == 'add-zone-user'):
			xml = CommandParser.parseAddZoneUser(args, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)

		elif (command == 'remove-zone-user'):
			xml = CommandParser.parseRemoveZoneUser(args, sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)

		elif (command == 'shutdown'):
			xml = CommandParser.parseShutdown(sessionId)
			responseXml = transport.sendRequest(xml)
			response = Response()
			response.parseXml(responseXml)
			return str(response)

		elif (command == 'help' or command == '?'):
			return CommandUsage.help()
	
		elif (command == 'quit' or command == 'bye' or command == 'exit'):
			self.quit()
		
		else:
			return "Invalid command: " + command

	def login(self, username, password):
		loginRequest = LoginRequest()
		loginRequest.username = username
		loginRequest.password = password
		xml = loginRequest.toXml()
		responseXml = self.transport.sendRequest(xml)
		loginResponse = LoginResponse()
		loginResponse.parseXml(responseXml)
		if (loginResponse.status == ResponseStatus.OK):
			self.sessionId = loginResponse.sessionId
			return True
		
		return False
	
	def quit(self):
		readline.set_history_length(100)
		readline.write_history_file(HISTORY_FILE)
		sys.exit()

	def run(self):
		if (self.username != '' and self.password != ''):
			loginStatus = self.login(self.username, self.password)
			if (loginStatus == True):
				print("Logged in as " + self.username)
			else:
				print("Login failed")
		try:
			readline.read_history_file(HISTORY_FILE)
		except IOError:
			pass
		
		while (True):
			try:
				userInput = input("dnssh> ")
			except EOFError:
				self.quit()
				
			userInput = userInput.strip()
			args = shlex.split(userInput)
			if (len(args) == 0):
				continue
		
			command = args[0].lower()
			try:
				response = self.parseCommand(command, args[1:])
				print(response)
			except CommandParserException as cpe:
				print
				if (cpe.msg != ''):
					print(cpe.msg)
					print
				CommandUsage.usage(command)
			except getopt.GetoptError:
				print
				print("Invalid syntax")
				print
				CommandUsage.usage(command)
			except TransportException:
				print("Connection to server failed")

			

def usage():
	print
	print("Usage:")
	print("  python dnssh.py [-h] [-s <server_IP>] [-p <server_port>]")
	print("                  [-u <username> -P <password>] ")
	print("  -h Print this help")
	print("  -s <server_IP>: Server's IP address (default: " + SERVER + ")")
	print("  -p <server_port>: Server's Port (default: " + str(PORT) + ")")
	print( "  -u <username>: Username to log into the server (Password must be given)")
	print("  -P <password>: Password used to log into the server")

if __name__ == "__main__":
	try:
		opts, args = getopt.getopt(sys.argv[1:], 'hs:p:u:P:')
	except:
		usage()
		sys.exit()
		
	server = SERVER
	port = PORT
	username = ''
	password = ''
	for opt, value in opts:
		if (opt == '-h'):
			usage()
			sys.exit()
		elif (opt == '-s'):
			server = value
		elif (opt == '-p'):
			try:
				port = int(value)
			except:
				usage()
				sys.exit()
		elif (opt == '-u'):
			username = value
		elif (opt == '-P'):
			password = value

	dnssh = Dnssh(server, port, username, password)
	dnssh.run()
