#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
import hashlib

from Cheetah.Template import Template
from .RestrictOperation import RestrictOperation;

class ChangePasswordRequest(RestrictOperation):
	"""Request object for the changePassword operation (change a user's password)."""
	TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<dnsshim version="1.0">
	<request>
		<changePassword>
			<sessionId>$request.sessionId</sessionId>
			<username>$request.username</username>
			<oldPassword>$request._ChangePasswordRequest__oldPasswordHash</oldPassword>
			<newPassword>$request._ChangePasswordRequest__newPasswordHash</newPassword>
		</changePassword>
	</request>
</dnsshim>
	"""
	"""XML template for the request"""
	
	def __init__(self):
		"""Default constructor"""
		self.username = ""
		"""Username"""
		self.oldPassword = ""
		"""User's current password in plain text"""
		self.newPassword = ""
		"""User's new password in plain text"""
		self.__oldPasswordHash = ""
		"""User's current password in SHA1 hash form"""
		self.__newPasswordHash = ""
		"""User's new password in SHA1 hash form"""

	def toXml(self):
		"""
		Generate the request XML
		@return: Request XML document
		"""
		sha = hashlib.sha1()
		sha.update(self.username)
		sha.update(self.oldPassword)
		self.__oldPasswordHash = sha.hexdigest()
		
		sha = hashlib.sha1()
		sha.update(self.username)
		sha.update(self.newPassword)
		self.__newPasswordHash = sha.hexdigest()
		
		template = Template(self.TEMPLATE)
		template.request = self
		return str(template)
