#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from Cheetah.Template import Template

from .DnskeyAlgorithm import DnskeyAlgorithm
from .DnskeyFlags import DnskeyFlags
from .DnskeyType import DnskeyType
from .DnskeyStatus import DnskeyStatus
from .RestrictOperation import RestrictOperation;

class ImportKeyRequest(RestrictOperation):
	"""Request object for the importKey operation (import a RSASHA1 DNSKEY into a zone )."""
	TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<dnsshim version="1.0">
	<request>
		<importKey>
			<sessionId>$request.sessionId</sessionId>
			<zone>$request.zone</zone>
			<algorithm>$request.algorithm</algorithm>
			<flags>$request.flags</flags>
			<type>$request.keyType</type>
			<status>$request.keyStatus</status>
			<modulus>$request.modulus</modulus>
			<publicExponent>$request.publicExponent</publicExponent>
			<privateExponent>$request.privateExponent</privateExponent>
			<prime1>$request.prime1</prime1>
			<prime2>$request.prime2</prime2>
			<exponent1>$request.exponent1</exponent1>
			<exponent2>$request.exponent2</exponent2>
			<coefficient>$request.coefficient</coefficient>
		</importKey>
	</request>
</dnsshim>
	"""
	"""XML template for the request"""
	
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		self.algorithm = DnskeyAlgorithm.RSASHA1
		"""DNSKEY algorithm (only RSASHA1 supported)"""
		self.flags = DnskeyFlags.SEP_ON
		"""DNSKEY flags attribute"""
		self.keyType = DnskeyType.ZSK
		"""Key type (ZSK or KSK)"""
		self.keyStatus = DnskeyStatus('PUBLISH')
		"""Key status (PUBLISH, SIGN or NONE)"""
		self.modulus = ""
		"""Modulus of the key"""
		self.publicExponent = ""
		"""Public exponent"""
		self.privateExponent = ""
		"""Private exponent"""
		self.prime1 = ""
		"""Prime1 parameter"""
		self.prime2 = ""
		"""Prime2 parameter"""
		self.exponent1 = ""
		"""Exponent1 parameter"""
		self.exponent2 = ""
		"""Exponent2 parameter"""
		self.coefficient = ""
		"""Coefficient parameter"""     

	def toXml(self):         
		"""
		Generate the request XML
		@return: Request XML document
		"""
		template = Template(self.TEMPLATE)
		template.request = self
		return str(template)
