#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from .Response import Response
from .ResponseStatus import ResponseStatus

from xml.dom.minidom import parseString
from io import StringIO

class ListKeysResponse(Response):
	"""Response object for the listKeys operation (list all the keys of a zone)."""
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		self.noneKeys = []
		"""List of keys with status NONE"""
		self.publishKeys = []
		"""List of keys with status PUBLISH"""
		self.signKeys = []
		"""List of keys with status SIGN"""
		
	def __str__(self):
		"""String representation of the object"""
		result = StringIO()
		result.write(Response.__str__(self) + "\n")
		if (self.status != ResponseStatus.OK):
			return result.getvalue()

		result.write("Zone: " + self.zone + "\n")
		
		result.write("Sign Keys:\n")
		for key in self.signKeys:
			result.write("\t" + key + "\n")
			
		result.write("Publish Keys:\n")
		for key in self.publishKeys:
			result.write("\t" + key + "\n")
			
		result.write("None Keys:\n")
		for key in self.noneKeys:
			result.write("\t" + key + "\n")
									
		return result.getvalue()
		
	def parseXml(self, xml):
		"""
		Parses a response XML document and fills the fields of the current object
		@param xml: Response XML document
		"""
		Response.parseXml(self, xml)
		if (self.status != ResponseStatus.OK):
			return
    
		dom = parseString(xml)
		node = dom.getElementsByTagName("listKeys")[0]
		for n in node.childNodes:
			if (n.nodeName == "zone"):
				self.zone = n.firstChild.data
			elif (n.nodeName == "keys"):
				for m in n.childNodes:
					if (m.nodeName == "none"):
						for k in m.childNodes:
							if (k.nodeName == "key"):
								self.noneKeys.append(k.firstChild.data)
					elif (m.nodeName == "publish"):
						for k in m.childNodes:
							if (k.nodeName == "key"):
								self.publishKeys.append(k.firstChild.data)						
					elif (m.nodeName == "sign"):
						for k in m.childNodes:
							if (k.nodeName == "key"):
								self.signKeys.append(k.firstChild.data)
								
								
