#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from .Response import Response
from .ResponseStatus import ResponseStatus
from .Slave import Slave

from xml.dom.minidom import parseString
from io import StringIO

class ListSlavesResponse(Response):
	"""Response object for the listSlaves operation (list all the slaves of a zone)."""
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		self.slaveGroups = {}
		"""Dictionary containing the slavegroup name as key and a list of slaves as value"""
		
	def __str__(self):
		"""String representation of the object"""
		result = StringIO()
		result.write(Response.__str__(self) + "\n")
		if (self.status != ResponseStatus.OK):
			return result.getvalue()

		result.write("Zone: " + self.zone + "\n")
		for slaveGroup in self.slaveGroups:
			result.write("Group: " + slaveGroup + "\n")
			for slave in self.slaveGroups[slaveGroup]:
				result.write("\t" + str(slave) + "\n")
		return result.getvalue()
		
	def parseXml(self, xml):
		"""
		Parses a response XML document and fills the fields of the current object
		@param xml: Response XML document
		"""
		Response.parseXml(self, xml)
		if (self.status != ResponseStatus.OK):
			return
    
		dom = parseString(xml)
		node = dom.getElementsByTagName("listSlaves")[0]
		for n in node.childNodes:
			if (n.nodeName == "zone"):
				self.zone = n.firstChild.data
				continue
			
			if (n.nodeName == "slaveGroups"):
				for slaveGroup in n.childNodes:
					if (slaveGroup.nodeName != "slaveGroup"):
						continue
					
					slaveGroupName = ""
					for groupNode in slaveGroup.childNodes:
						if (groupNode.nodeName == "name"):
							slaveGroupName = groupNode.firstChild.data
							continue
						
						if (groupNode.nodeName == "slaves"):
							slaves = []
							for slave in groupNode.childNodes:
								if (slave.nodeName != "slave"):
									continue
								
								address = ""
								port = 53
								for slaveNode in slave.childNodes:
									if (slaveNode.nodeName == "address"):
										address = slaveNode.firstChild.data
										continue
									
									if (slaveNode.nodeName == "port"):
										port = int(slaveNode.firstChild.data)
								slave = Slave(address, port)
								slaves.append(slave)
					
					self.slaveGroups[slaveGroupName] = slaves
					
