#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from .Response import Response
from .ResponseStatus import ResponseStatus
from .DsDigestType import DsDigestType

from xml.dom.minidom import parseString
from io import StringIO

class NewKeyResponse(Response):
	"""Response object for the newKey operation (Generate a new DNSKEY for a zone)."""
	def __init__(self):
		"""Default constructor"""
		Response()
		self.zone = ""
		"""Zonename"""
		self.keyName = ""
		"""Name of the new key in the DNSSHIM"""
		self.keytag = 0
		"""Keytag of the new key"""
		self.digest = ""
		"""DS digest"""
		self.digestType = DsDigestType.SHA1
		"""DS digest algorithm (default is SHA1)"""
    
	def __str__(self):
		"""String representation of the object"""
		result = StringIO()
		result.write(Response.__str__(self) + "\n")
		if (self.status != ResponseStatus.OK):
			return result.getvalue()

		result.write("NewKey:\n")
		result.write("\tZone: " + self.zone + "\n")
		result.write("\tKey Name: " + self.keyName + "\n")
		result.write("\tKeytag: " + str(self.keytag) + "\n")
		result.write("\tDS digest: " + self.digest + "\n")
		result.write("\tDigest Type: " + str(self.digestType) + "\n")
		return result.getvalue()
    
	def parseXml(self, xml):
		"""
		Parses a response XML document and fills the fields of the current object
		@param xml: Response XML document
		"""
		Response.parseXml(self, xml)
		if (self.status != ResponseStatus.OK):
			return
    
		dom = parseString(xml)

		keyNode = dom.getElementsByTagName("newKey")[0]
		for childNode in keyNode.childNodes:
			nodeName = childNode.nodeName
			if (nodeName == "zone"):
				zone = childNode.firstChild.data
				self.zone = zone
      
			elif (nodeName == "name"):
				keyName = childNode.firstChild.data
				self.keyName = keyName
        
			elif (nodeName == "keytag"):
				keytag = childNode.firstChild.data
				self.keytag = keytag
      
			elif (nodeName == "digest"):
				digest = childNode.firstChild.data
				self.digest = digest
        
			elif (nodeName == "digestType"):
				digestType = childNode.firstChild.data
				self.digestType = digestType
  
