#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
 
from Cheetah.Template import Template
from .DnskeyAlgorithm import DnskeyAlgorithm
from .DnskeyFlags import DnskeyFlags
from .DnskeyProtocol import DnskeyProtocol
from .DnskeyStatus import DnskeyStatus
from .DnskeyType import DnskeyType
from .RestrictOperation import RestrictOperation

class NewZoneRequest(RestrictOperation):
	"""Request object for the newZone operation (create a new zone)."""
	TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<dnsshim version="1.0">
	<request>
		<newZone>
			<sessionId>$request.sessionId</sessionId>
			<zone>$request.zone</zone>
			<dnssec>$request.isSigned()</dnssec>
			#if $request.slaveGroup != ""
			<slaveGroup>$request.slaveGroup</slaveGroup>			
			#end if
			#if $request.isAutoBalance()
			<autoBalance>true</autoBalance>			
			#end if
			#if $request.isVerifySignatures()
			<verifySignatures>true</verifySignatures>			
			#end if
			<soa>
				<ttl>$request.ttl</ttl>
				<mname>$request.mname</mname>
				<rname>$request.rname</rname>
				<serial>$request.serial</serial>
				<refresh>$request.refresh</refresh>
				<retry>$request.retry</retry>
				<expire>$request.expire</expire>
				<minimum>$request.minimum</minimum>
			</soa>
			#if $request.isSigned() == 1
			<key>
				<size>$request.keySize</size>
				<algorithm>$request.algorithm</algorithm>
				<expirationPeriod>$request.expirationPeriod</expirationPeriod>
			</key>
			#end if
		</newZone>
	</request>
</dnsshim>
	"""
	"""XML template for the request"""
	
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		self.ttl = 86400
		"""SOA TTL (default is 86400)"""
		self.mname = ""
		"""SOA MNAME"""
		self.rname = ""
		"""SOA RNAME"""
		self.serial = 1
		"""SOA SERIAL number (default is 1)"""
		self.refresh = 86400
		"""SOA REFRESH field (default is 86400)"""
		self.retry = 900
		"""SOA RETRY field (default is 900)"""
		self.expire = 604800
		"""SOA EXPIRE field (default is 604800)"""
		self.minimum = 900
		"""SOA MINIMUM field (default is 900)"""
		self.signed = True
		"""Whether the zone is signed or not (default is TRUE)""" 
		self.keySize = 0
		"""Keysize of the zone's DNSKEY"""
		self.algorithm = DnskeyAlgorithm.ECDSAP256SHA256
		"""Algorithm of the zone's DNSKEY (default is ECDSA (13))"""
		self.expirationPeriod = 2592000 # 1 month
		"""Validity period of the zone's signatures (default is 2592000 or 1 month)"""
		self.slaveGroup = ""
		"""SLAVE GROUP NAME"""
		self.autoBalance = False
		"""Auto balance slave groups"""
		self.verifySignatures = False
		"""Verify Signatures"""

	def toXml(self):
		"""
		Generate the request XML
		@return: Request XML document
		"""
		template = Template(self.TEMPLATE)
		template.request = self
		return str(template)
	
	def isSigned(self):
		return int(self.signed)

	def isAutoBalance(self):
		return int(self.autoBalance)
        
	def isVerifySignatures(self):
		return int(self.verifySignatures)
