#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from .DsInfo import DsInfo
from .Response import Response
from .ResponseStatus import ResponseStatus
from .ResourceRecord import ResourceRecord

from xml.dom.minidom import parseString
from io import StringIO

class PrintZoneResponse(Response):
	"""Response object for the printZone operation (print a zone's Resource Records)."""
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		self.records = []
		"""List of Resource Records in the zone"""
		self.dsInfo = []
		"""List of DS information for the DNSKEYs in the zone"""
		
	def __str__(self):
		"""String representation of the object"""
		result = StringIO()
		result.write(Response.__str__(self))
		if (self.status != ResponseStatus.OK):
			return result.getvalue()
		
		result.write("Zone: " + self.zone + "\n")

		for rr in self.records:
			result.write("\t" + str(rr) + "\n")
		
		result.write("\nDS Info:\n")
		for ds in self.dsInfo:
			result.write("\t" + str(ds) + "\n")
			
		return result.getvalue()
		
	def parseXml(self, xml):
		"""
		Parses a response XML document and fills the fields of the current object
		@param xml: Response XML document
		"""
		Response.parseXml(self, xml)
		if (self.status != ResponseStatus.OK):
			return
   
		dom = parseString(xml)
		node = dom.getElementsByTagName("printZone")[0]
		for n in node.childNodes:
			nodeName = n.nodeName
			if (nodeName == "zone"):
				zone = n.firstChild.data
				self.zone = zone
			
			elif (nodeName == "records"):
				for rrNode in n.childNodes:
					if (rrNode.nodeName != "rr"):
						continue
					
					rr = ResourceRecord()
					for field in rrNode.childNodes:
						if (field.nodeName == "ownername"):
							if field.firstChild == None:
								rr.ownername = ""
							else:
								rr.ownername = field.firstChild.data
						elif (field.nodeName == "class"):
							rr.dnsClass = field.firstChild.data
						elif (field.nodeName == "type"):
							rr.type = field.firstChild.data
						elif (field.nodeName == "ttl"):
							rr.ttl = field.firstChild.data
						elif (field.nodeName == "rdata"):
							rr.rdata = field.firstChild.data
							
					self.records.append(rr)
			elif (nodeName == "dsInfo"):
				for dsNode in n.childNodes:
					if (dsNode.nodeName != "ds"):
						continue
					
					dsInfo = DsInfo()
					for field in dsNode.childNodes:
						if (field.nodeName == "keytag"):
							dsInfo.keytag = field.firstChild.data
						elif (field.nodeName == "digestType"):
							dsInfo.digestType = field.firstChild.data
						elif (field.nodeName == "digest"):
							dsInfo.digest = field.firstChild.data
					
					self.dsInfo.append(dsInfo)
				
