#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from .Response import Response
from .ResponseStatus import ResponseStatus

from io import StringIO
from xml.dom.minidom import parseString

class RemoveZoneResponse(Response):
	"""Response object for the removeZone operation (remove a zone from DNSSHIM)."""
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		
	def __str__(self):
		"""String representation of the object"""
		result = StringIO()
		result.write(Response.__str__(self) + "\n")
		if (self.status != ResponseStatus.OK):
			return result.getvalue()

		result.write("RemoveZone: " + self.zone + "\n")
		return result.getvalue()
	
	def parseXml(self, xml):
		"""
		Parses a response XML document and fills the fields of the current object
		@param xml: Response XML document
		"""
		Response.parseXml(self, xml)
		if (self.status != ResponseStatus.OK):
			return

		dom = parseString(xml)
		node = dom.getElementsByTagName("removeZone")[0]
		for n in node.childNodes:
			if (n.nodeName == "zone"):
				self.zone = n.firstChild.data
