#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from Cheetah.Template import Template
from .RestrictOperation import RestrictOperation

class SetExpirationPeriodRequest(RestrictOperation):
	"""Request object for the setExpirationPeriod operation (set the signatures' validity period of zone)."""
	TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<dnsshim version="1.0">
	<request>
		<setExpirationPeriod>
			<sessionId>$request.sessionId</sessionId>
			<zone>$request.zone</zone>
			<expirationPeriod>$request.expiration</expirationPeriod>
		</setExpirationPeriod>
	</request>
</dnsshim>
	"""
	"""XML template for the request"""
	
	def __init__(self):
		"""Default constructor"""
		self.zone = ""
		"""Zonename"""
		self.expiration = 2592000
		"""The zone's signatures validity period (default is 2592000 or 1 month)"""
		
	def toXml(self):
		"""
		Generate the request XML
		@return: Request XML document
		"""
		template = Template(self.TEMPLATE)
		template.request = self
		return str(template)
