#  Copyright (C) 2009 Registro.br. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 1. Redistribution of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
 
from Cheetah.Template import Template

from .SlaveOperation import SlaveOperation
from .RestrictOperation import RestrictOperation
from .RrOperation import RrOperation

class SlaveRequest(RestrictOperation):
	"""Request object for the addSlave and removeSlave operations (add and remove a slave to and from a slavegroup)."""
	ADD_SLAVE_TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<dnsshim version="1.0">
	<request>
		<addSlave>
			<sessionId>$request.sessionId</sessionId>
			<slaveGroup>$request.slaveGroup</slaveGroup>
			<slave>$request.slave</slave>
			<port>$request.port</port>
		</addSlave>
	</request>
</dnsshim>
	"""
	"""XML template for the addSlave request"""
	
	REMOVE_SLAVE_TEMPLATE = """
<?xml version="1.0" encoding="UTF-8"?>
<dnsshim version="1.0">
	<request>
		<removeSlave>
			<sessionId>$request.sessionId</sessionId>
			<slaveGroup>$request.slaveGroup</slaveGroup>
			<slave>$request.slave</slave>
			<port>$request.port</port>
		</removeSlave>
	</request>
</dnsshim>
	"""
	"""XML template for the removeSlave request"""
	
	def __init__(self):
		"""Default constructor"""
		self.operation = SlaveOperation.ADD
		"""Operation to be executed: ADD or REMOVE"""
		self.slaveGroup = ""
		"""Slavegroup name"""
		self.slave = ""
		"""IP address of the slave server being added or removed"""
		self.port = 53
		"""Port number of the slave server (default is 53)"""

	def toXml(self):         
		"""
		Generate the request XML
		@return: Request XML document
		"""
		if (self.operation == SlaveOperation.ADD):
			template = Template(self.ADD_SLAVE_TEMPLATE)
		else:
			template = Template(self.REMOVE_SLAVE_TEMPLATE)
		template.request = self
		return str(template)
